#========================================================================================
# package      DE_EPAGES::DocdataPay::t::unit::LineItemPaymentDocdataPay
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for LineItemPaymentDocdataPay.
#========================================================================================
package DE_EPAGES::DocdataPay::t::unit::LineItemPaymentDocdataPay;

use base DE_EPAGES::Core::API::Test;
use strict;

use DE_EPAGES::DocdataPay::API::Object::LineItemPaymentDocdataPay;
use DE_EPAGES::DocdataPay::API::Object::PaymentMethodDocdataPay;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Readonly;
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;

#========================================================================================
sub test {
    my $self = shift;
    $self->_test_status_totalAcquirerApproved_paid_api12();
    $self->_test_status_totalCaptured_paid_api12();
    $self->_test_status_canceled_api12();
    $self->_test_status_authorized_api12();
    $self->_test_set_status_approved();
    $self->_test_set_status_cancelled();
    $self->_test_capture();
    done_testing();
}

#========================================================================================
sub _test_status_totalAcquirerApproved_paid_api12 {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedLineItemPaymentDocdataPay = $self->_createLineItemPaymentDocdataPay('totalAcquirerApproved_PAID');
    $MockedLineItemPaymentDocdataPay->getTransStatus();

    is( $MockedLineItemPaymentDocdataPay->get('TransStatus'), 'STATUS_PAID', "$SubTestName - Status is STATUS_PAID" );
    is(
        $MockedLineItemPaymentDocdataPay->get('TransTimeMod'),
        'Feb 23, 2015 11:44:41 AM',
        "$SubTestName - TransTimeMod has been successfully updated."
    );
    is(
        $MockedLineItemPaymentDocdataPay->get('TransTime'),
        'Feb 23, 2015 11:44:41 AM',
        "$SubTestName - TransTime has been successfully updated."
    );

    return;
}

#========================================================================================
sub _test_status_totalCaptured_paid_api12 {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedLineItemPaymentDocdataPay = $self->_createLineItemPaymentDocdataPay('totalCaptured_PAID');
    $MockedLineItemPaymentDocdataPay->getTransStatus();

    is( $MockedLineItemPaymentDocdataPay->get('TransStatus'), 'STATUS_PAID', "$SubTestName - Status is STATUS_PAID" );
    is(
        $MockedLineItemPaymentDocdataPay->get('TransTimeMod'),
        'Feb 23, 2015 11:44:41 AM',
        "$SubTestName - TransTimeMod has been successfully updated."
    );
    is(
        $MockedLineItemPaymentDocdataPay->get('TransTime'),
        'Feb 23, 2015 11:44:41 AM',
        "$SubTestName - TransTime has been successfully updated."
    );

    return;
}

#========================================================================================
sub _test_status_authorized_api12 {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedLineItemPaymentDocdataPay = $self->_createLineItemPaymentDocdataPay('AUTHORIZED');
    $MockedLineItemPaymentDocdataPay->getTransStatus();

    is( $MockedLineItemPaymentDocdataPay->get('TransStatus'), 'STATUS_STARTED', "$SubTestName - Status is STATUS_STARTED" );
    is(
        $MockedLineItemPaymentDocdataPay->get('TransTimeMod'),
        'Feb 23, 2015 11:44:41 AM',
        "$SubTestName - TransTimeMod has been successfully updated."
    );
    is(
        $MockedLineItemPaymentDocdataPay->get('TransTime'),
        'Feb 23, 2015 11:44:41 AM',
        "$SubTestName - TransTime has been successfully updated."
    );

    return;
}

#========================================================================================
sub _test_status_canceled_api12 {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedLineItemPaymentDocdataPay = $self->_createLineItemPaymentDocdataPay('CANCELED');
    $MockedLineItemPaymentDocdataPay->getTransStatus();

    is( $MockedLineItemPaymentDocdataPay->get('TransStatus'), 'STATUS_FAILED', "$SubTestName - Status is STATUS_FAILED" );
    is(
        $MockedLineItemPaymentDocdataPay->get('TransTimeMod'),
        'Feb 23, 2015 11:44:41 AM',
        "$SubTestName - TransTimeMod has been successfully updated."
    );
    is(
        $MockedLineItemPaymentDocdataPay->get('TransTime'),
        'Feb 23, 2015 11:44:41 AM',
        "$SubTestName - TransTime has been successfully updated."
    );

    return;
}

#========================================================================================
sub _test_set_status_approved {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedLineItemPaymentDocdataPay = $self->_createLineItemPaymentDocdataPay_withDate('totalAcquirerApproved_PAID', 'Feb 23, 2015 11:44:41 AM');
    $MockedLineItemPaymentDocdataPay->setOrderStatus('PaidOn');

    is(
        $MockedLineItemPaymentDocdataPay->parent()->parent()->get('PaidOn'),
        'Feb 23, 2015 11:44:41 AM',
        "$SubTestName - Order status has been successfully updated.");

    return;
}

#========================================================================================
sub _test_set_status_cancelled {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedLineItemPaymentDocdataPay = $self->_createLineItemPaymentDocdataPay_withDate('CANCELED', 'Feb 23, 2015 11:44:41 AM');
    $MockedLineItemPaymentDocdataPay->setOrderStatus('CancelledOn');

    is(
        $MockedLineItemPaymentDocdataPay->parent()->parent()->get('CancelledOn'),
        'Feb 23, 2015 11:44:41 AM',
        "$SubTestName - Order status has been successfully updated.");

    return;
}

#========================================================================================
sub _test_capture {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedLineItemPaymentDocdataPay = $self->_createLineItemPaymentDocdataPay();
    my $hParsedResponse;
    lives_ok { $hParsedResponse = $MockedLineItemPaymentDocdataPay->capturePayment() }
    "$SubTestName - Successful capture request.";
    is( $hParsedResponse, undef, "$SubTestName - Successful capture request returns nothing." );

    return;
}

#========================================================================================
#   Wrappers
#========================================================================================
sub _createLineItemPaymentDocdataPay {
    my $self = shift;
    my ( $PaymentStatus ) = @_;
    return $self->_createLineItemPaymentDocdataPay_withDate($PaymentStatus, undef);
}
#========================================================================================
sub _createLineItemPaymentDocdataPay_withDate {
    my $self = shift;
    my ( $PaymentStatus , $Date) = @_;

    my $LineItemPaymentDocdataPay       = DE_EPAGES::DocdataPay::API::Object::LineItemPaymentDocdataPay->new();
    my $MockedLineItemPaymentDocdataPay = Test::MockObject::Extends->new($LineItemPaymentDocdataPay);
    $MockedLineItemPaymentDocdataPay->{'Attributes'}->{'APIVersion'}        = '1.2';
    $MockedLineItemPaymentDocdataPay->{'Attributes'}->{'PaymentID'}         = '4907435245';
    $MockedLineItemPaymentDocdataPay->{'Attributes'}->{'PaymentMethod'}     = $self->_createPaymentMethodDocdataPay();
    $MockedLineItemPaymentDocdataPay->{'Attributes'}->{'IsTestTransaction'} = 1;
    $MockedLineItemPaymentDocdataPay->{'Parent'}                            = $self->_createLineItemContainer();
    $MockedLineItemPaymentDocdataPay->{'Attributes'}->{'TransStatus'}       = undef;
    $MockedLineItemPaymentDocdataPay->{'Attributes'}->{'TransTimeMod'}      = undef;
    $MockedLineItemPaymentDocdataPay->{'Attributes'}->{'TransTime'}         = $Date;

    $MockedLineItemPaymentDocdataPay->mock(
        '_sendRequest',
        sub {
            shift;
            my ( $RequestType ) = @_;

            my $hResponse;

            if ( $RequestType eq 'status' ) {
                if ($PaymentStatus eq 'totalAcquirerApproved_PAID') {
                    $hResponse = {
                        'totalRegistered'       => '100',
                        'totalCaptured'         => '0',
                        'totalAcquirerApproved' => '100',
                        'authorizationStatus'   => 'PAID'
                    };
                } elsif ($PaymentStatus eq 'totalCaptured_PAID') {
                    $hResponse = {
                        'totalRegistered'       => '100',
                        'totalCaptured'         => '100',
                        'totalAcquirerApproved' => '0',
                        'authorizationStatus'   => 'PAID'
                    };
                } elsif ($PaymentStatus eq 'AUTHORIZED') {
                    $hResponse = {
                        'totalRegistered'       => '100',
                        'totalCaptured'         => '0',
                        'totalAcquirerApproved' => '80',
                        'authorizationStatus'   => 'AUTHORIZED'
                    };
                } elsif ($PaymentStatus eq 'CANCELED') {
                    $hResponse = {
                        'totalRegistered'       => '100',
                        'totalCaptured'         => '0',
                        'totalAcquirerApproved' => '0',
                        'authorizationStatus'   => 'CANCELED'
                    };
                }
                return $hResponse;
            }
        });

    $MockedLineItemPaymentDocdataPay->mock('set', sub {
        shift;
        my ( $hLineItemPaymentParameters ) = @_;

        foreach my $key ( keys %$hLineItemPaymentParameters ){
            $MockedLineItemPaymentDocdataPay->{'Attributes'}->{$key} = $hLineItemPaymentParameters->{$key}
        }
        return;
    });

    $MockedLineItemPaymentDocdataPay->mock('_TestObject', sub {
        shift;
        return;
    });

    $MockedLineItemPaymentDocdataPay->mock('_isValid', sub {
        shift;
        return;
    });

    $MockedLineItemPaymentDocdataPay->mock('_getCurrentDateTime', sub {
        shift;
        return 'Feb 23, 2015 11:44:41 AM';
    });

    return $MockedLineItemPaymentDocdataPay;
}

#========================================================================================
sub _createPaymentMethodDocdataPay {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::DocdataPay::API::Object::PaymentMethodDocdataPay',
        $aParameterList );

    return $Mockify->getMockObject();
}

#========================================================================================
sub _createLineItemContainer {
    my $self = shift;
    my ( $BasketOrOrder ) = @_;

    my $hAtributes = { 'Parent' => $self->_createCustomerOrder() };

    my $LineItemContainer =
      DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer', $hAtributes );

    return $LineItemContainer;
}

#========================================================================================
sub _createCustomerOrder {
    my $self = shift;

    my $hAtributes = { 'PaidOn'       => undef,
                       'CancelledOn'  => undef };
    my $CustomerOrder =
        DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::CustomerOrder', $hAtributes );

    return $CustomerOrder;
}

__PACKAGE__->runScript();
