#========================================================================================
# package      DE_EPAGES::Test::t::unit::Mockify_MethodCallCounter
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit tests for Mockify_MethodCallCounter
#========================================================================================
package DE_EPAGES::Test::t::unit::Mockify_MethodCallCounter;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Test::Mock::Mockify::MethodCallCounter;
use Test::More;
use Test::Exception;
use strict;

sub test {
    my $self = shift;

    $self->test_MethodCallCounter_oneCall_positive();
    $self->test_MethodCallCounter_multipleCalls_positive();
    $self->test_MethodCallCounter_getAmountOfCalls_negative();
    $self->test_MethodCallCounter_increment_negative();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_MethodCallCounter_oneCall_positive {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MethodCallCounter = DE_EPAGES::Test::Mock::Mockify::MethodCallCounter->new();
    my $MethodName = 'TestMethod';
    $MethodCallCounter->addMethod($MethodName);

    $MethodCallCounter->increment($MethodName);
    my $Result = $MethodCallCounter->getAmountOfCalls($MethodName);

    is( $Result, 1, "$SubTestName - tests if the increment is count to 1" );
}
#----------------------------------------------------------------------------------------
sub test_MethodCallCounter_multipleCalls_positive {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MethodCallCounter = DE_EPAGES::Test::Mock::Mockify::MethodCallCounter->new();
    my $MethodName = 'TestMethod';
    $MethodCallCounter->addMethod($MethodName);
    $MethodCallCounter->addMethod('otherMethodName');

    $MethodCallCounter->increment($MethodName);
    $MethodCallCounter->increment($MethodName);
    $MethodCallCounter->increment('otherMethodName');
    $MethodCallCounter->increment($MethodName);
    my $Result = $MethodCallCounter->getAmountOfCalls($MethodName);

    is( $Result, 3, "$SubTestName - tests if the increment is count to 3" );
}
#----------------------------------------------------------------------------------------
sub test_MethodCallCounter_getAmountOfCalls_negative {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MethodCallCounter = DE_EPAGES::Test::Mock::Mockify::MethodCallCounter->new();
    my $MethodName = 'TestMethod';
    throws_ok( sub { $MethodCallCounter->getAmountOfCalls($MethodName) },
               qr/The Method: 'TestMethod' was not added to Mockify/,## no critic (ProhibitComplexRegexes ProhibitEscapedMetacharacters)
               "$SubTestName - test the Error if the Method was not added into mockify"
     );
}
#----------------------------------------------------------------------------------------
sub test_MethodCallCounter_increment_negative {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MethodCallCounter = DE_EPAGES::Test::Mock::Mockify::MethodCallCounter->new();
    my $MethodName = 'TestMethod';
    throws_ok( sub { $MethodCallCounter->increment($MethodName) },
               qr/The Method: 'TestMethod' was not added to Mockify/,## no critic (ProhibitComplexRegexes ProhibitEscapedMetacharacters)
               "$SubTestName - test the Error if the Method was not added into mockify"
     );
}

__PACKAGE__->runScript();