#========================================================================================
# package      DE_EPAGES::Test::t::unit::Mockify_ServletVarsService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for ServletVarsService
#========================================================================================
package DE_EPAGES::Test::t::unit::Mockify_ServletVarsService;
use base DE_EPAGES::Core::API::Test;
use Test::More;
use Test::Exception;
use strict;
use DE_EPAGES::Test::Mock::Mockify::ServletVarsService;

sub test {
    my $self = shift;

    $self->test_getAllVariables();
    $self->test_getAVariable();
    $self->test_getAVariable_WrongKey();
    $self->test_setAVariable();

    $self->test_setVarValidator();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_getAllVariables {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ServletVarsService = DE_EPAGES::Test::Mock::Mockify::ServletVarsService->new( );
    $ServletVarsService->addVar('Key1','Value1');
    $ServletVarsService->addVar(
            'Key2',
            {
                'more'=>'complex',
                'Struc'=>['t','u','r','e']
            }
    );

    my $hResult = $ServletVarsService->vars();

    my $hExpectedVars = {
        'Key1' => 'Value1',
        'Key2' => {
            'more'=>'complex',
            'Struc'=>['t','u','r','e']
        },
    };
    is_deeply( $hResult, $hExpectedVars, "$SubTestName - tests if all expected variables are there" );
}
#----------------------------------------------------------------------------------------
sub test_getAVariable {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ServletVarsService = DE_EPAGES::Test::Mock::Mockify::ServletVarsService->new( );
    $ServletVarsService->addVar( 'Key', 'Value' );

    my $Result = $ServletVarsService->vars( 'Key' );

    is( $Result, 'Value', "$SubTestName - tests if the value from key is there" );
}
#----------------------------------------------------------------------------------------
sub test_getAVariable_WrongKey {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ServletVarsService = DE_EPAGES::Test::Mock::Mockify::ServletVarsService->new( );

     throws_ok( sub { $ServletVarsService->vars('UserName') },
               qr/UnInitializedVariable: key=UserName/,## no critic (ProhibitComplexRegexes ProhibitEscapedMetacharacters)
               "$SubTestName - test the Error if method is called if the value is wrong"
     );
}
#----------------------------------------------------------------------------------------
sub test_setAVariable {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ServletVarsService = DE_EPAGES::Test::Mock::Mockify::ServletVarsService->new( );
    $ServletVarsService->addVar( 'OtherKey', 'OtherValue' );
    $ServletVarsService->addVar( 'Key', 'Value' );

    my $OldResult = $ServletVarsService->vars( 'Key' );
    my $SetterResult = $ServletVarsService->vars( 'Key' , 'NewValue' );
    my $NewResult = $ServletVarsService->vars( 'Key' );
    my $OtherKeyResult = $ServletVarsService->vars( 'OtherKey' );

    is( $OldResult, 'Value', "$SubTestName - tests if the old value was there" );
    is( $SetterResult, 'NewValue', "$SubTestName - tests if the return value of the setter-call is there" );
    is( $NewResult, 'NewValue', "$SubTestName - tests if the new value is set" );
    is( $OtherKeyResult, 'OtherValue', "$SubTestName - tests if the other values stays untouched" );
}
#----------------------------------------------------------------------------------------
sub test_setVarValidator {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ServletVarsService = DE_EPAGES::Test::Mock::Mockify::ServletVarsService->new( );

    $ServletVarsService->setVarValidator( 'Key1','string' );
    $ServletVarsService->setVarValidator( 'Key2',{'string'=>'abcd'} );
    my $ValidatorKey1 = $ServletVarsService->getVarValidator( 'Key1' );
    my $ValidatorKey2 = $ServletVarsService->getVarValidator( 'Key2' );
    my $ValidatorUndefinedKey = $ServletVarsService->getVarValidator( 'UndefinedKey' );

    is( $ValidatorKey1, 'string', "$SubTestName - tests if getter is working" );
    is_deeply( $ValidatorKey2, {'string'=>'abcd'}, "$SubTestName - tests if setter is working" );
    is( $ValidatorUndefinedKey, undef, "$SubTestName - tests if getter is working" );
}

__PACKAGE__->runScript();